/*
   The functions in this file test your recursive insertion sort function
   from the file RecursiveInsertion.h.

   These test functions fill an array of size n, where n is a command line parameter,
   with a random permutation of the integers from 1 to n, and then the random array
   is sorted. The time it takes to sort the array is (roughly) calculated.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "RecursiveInsertion.h"


/*
   This function will randomly permute the elements of the array.
*/
void permutation(int* a, int n)
{
   int i, upperBound, temp;

   srand( time(NULL) );

   upperBound = n;
   // The array from indices 0 to upperBound-1 is the "jar" to draw randomly
   // from, and the array from indices upperBound to n-1 is the list of
   // numbers that have already been drawn out of the "jar".
   for (i = 0; i < n; i++)
   {
      // pick a slot in the "jar" part of the array
      int randomIndex = rand() % upperBound;

      // swap a[upperBound-1] with a[randomIndex]
      // (that is, move an item from the "jar" part of the array
      // to the list part of the array)
      temp = a[upperBound-1];
      a[upperBound-1] = a[randomIndex];
      a[randomIndex] = temp;

      // lower the upperbound
      upperBound--;  // lower the boundary of the "jar" part of the array
   }
}//permutation()


/*
   This function prints an array in rows of 10.
*/
void printArray(int* a, int n)
{
   int i;
   printf("\nHere is the array.\n");
   for (i = 0; i < n; i++)
   {
      printf(" %d", a[i]);
      if ((i+1)%10 == 0) printf("\n");
   }
   printf("\n");
}//printArray()




int main(int argc, char** argv)
{
   time_t start,end;
   double dif;

   int i;

   int n;
   if (argc > 1)
   {
      n = atoi( argv[1] );  // get parameter from command line
   }
   else
   {
      printf("Usage: sort <n>");
      return 0;
   }

   int* integerArray = malloc(n*sizeof(int));  // array that will hold the permutation.

   // initialize the array with the numbers for 1 to n
   for (i = 0; i < n; i++) integerArray[i] = i+1;

   // the permutation() function will randomly permute
   // the contents of the array
   permutation(integerArray, n);

   // as soon as permutation() is done, print a message.
   printf("\nPermutation done.\n");

   // if n is small enough, print out the array.
   if (n <= 128)
   {
      printArray(integerArray, n);
   }

   // sort the array
   time (&start);
   recursiveInsertionSort( integerArray, 0, n-1 );
   time (&end);

   // as soon as sorting is done, print a message.
   printf("\nSort done.\n");
   dif = difftime (end, start);
   printf ("It took %.2lf seconds to sort the array.\n", dif );

   // if n is small enough, print out the array.
   if (n <= 128)
   {
      printArray(integerArray, n);
   }

   return 0;
}//main()
